/*
 * This sketch is controling a RGB123 board (16x16) with a RFDuino Arduino module 
 * This last is a Arm-Cortex M0 @ 16Mhz
 * Due to really short timing of WS2812b leds running under 1us the update procedure has to be 
 * written in assembly language and optimized for this purpose
 * 
 * ** This sketch only works with this specific Arduino board (or eventually Cortex-m0 @ 16Mhz others) **
 *
 *
 * Copyright (c) 2014 Paul Pinault www.disk91.com .  All right reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * Author : Paul Pinault - www.disk91.com
 *          follow me on twitter : @disk_91
 */

#include <RFduinoBLE.h>

// GPIO2 heartbit
int led1 = 6;
// GPIO1
int toRGB = 2;


void setup() {
  // setup the heartbit leds for output
  pinMode(led1, OUTPUT);
  rgb123_256_init(toRGB);
}

void loop() {
  int v = 1;
  for ( int y = 0 ; y < 16 ; y ++ ) {
    digitalWrite(led1, v);  
    v = v ^ 1;
    for ( int x = 0 ; x < 18 ; x++ ) {
      rgb123_256_setPixel(x, y, 0x85, 0, 0); 
      rgb123_256_setPixel(x-1, y, 0, 0x85, 0); 
      rgb123_256_setPixel(x-2, y, 0, 0, 0x85); 
      rgb123_256_showPixels(toRGB);
      delay(100);               // wait for a second
      rgb123_256_clrPixels();
    }
  }
}


// ---------------------------------------------------
// Raw data representing pixels each value format is
// --------bbbbbbbbrrrrrrrrgggggggg
// --------012345670123456701234567
// led matrix is 16x16, direction change each line
uint32_t rgb123_256_pixels[256];

// ---------------------------------------------------
// set a pixel in the coordinate [0-15][0-15]
void rgb123_256_setPixel(uint32_t x, uint32_t y, uint32_t r, uint32_t g, uint32_t b) {
  if ( x > 15 || y > 15 ) return;
  
  int v = 0;
  for ( int i = 0 ; i < 8 ; i++ ) 
    v = ( v | (( b >> i ) & 1) ) << 1;
  for ( int i = 0 ; i < 8 ; i++ ) 
    v = ( v | (( r >> i ) & 1) ) << 1;
  for ( int i = 0 ; i < 8 ; i++ ) 
    v = ( v | (( g >> i ) & 1) ) << 1;
  v = v >> 1;
  
  uint32_t _x = (y & 1 != 0)?15-x:x;
  rgb123_256_pixels[16*y+_x] = v;
}

// ---------------------------------------------------
// clear the pixel matrix
void rgb123_256_clrPixels() {
  for (int i = 0 ; i < 256 ; i++) rgb123_256_pixels[i] = 0;
}


// ---------------------------------------------------
// display pixel matrix
// max = 1 << 24                                    
// end = _ptbits + 256 * 4 + 4                      
// while ( _ptbits != end ) {      
//   v = *_ptbits                                   
//      b = v & 1
//      if (b) write(1) else write(0) ; v = v << 1
//      if (b) write(1) else write(0) ; v = v << 1
//      ... x24 ...
//      if (b) write(1) else write(0) ; v = v << 1
//   _ptbits++;
// }
void rgb123_256_showPixels(uint32_t pin) {
  volatile uint32_t * outset = &NRF_GPIO->OUTSET;
  volatile uint32_t * outclr = &NRF_GPIO->OUTCLR;
  uint32_t * _ptbits = rgb123_256_pixels;
  uint32_t * _end    = &rgb123_256_pixels[256];   
  uint32_t   _max    = ( 1 << 24 );
  uint32_t   _p      = 1 << pin;


  while (RFduinoBLE.radioActive)
     ;
  RFduinoBLE.end();
  noInterrupts();
  
  // outset  = %0
  // outclr  = %1
  // _ptbits = %2
  // _end    = %3
  // _max    = %4
  // _p      = %5
  //
  // v       = r7 %6
  // i       = r6 %7
  // b       = r5 %8
  int __v;
  int __i;
  int __b;
  asm volatile (
      "headSP:\n\t"
      "push {r0,r1,r2,r3,r4,r5,r6,r7}\n\t"
      "wh01: \n\t"                // while ( _ptbits < _end ) {
         "cmp %2,%3 \n\t"         //    1 - cycle
         "blo wh01s \n\t"         //    3 - cycles
         "b   ewh01 \n\t"

       "wh01s: \n\t"  
         "ldr %6,[%2,#0] \n\t"    //    v = *ptbits
         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"      
       "wh01_optim: \n\t"          // optimized loop return from end
         "str %5,[%0,#0] \n\t"  //    *outset = _p 
         "cmp %7,#0 \n\t"      
         "beq b0zero \n\t"
         "b0one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr  %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b0end \n\t"         //    -- cycle 1-2-3
         "b0zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b0end: \n\t"             //    -- 4 more cycle before high
        
         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b1zero \n\t"
         "b1one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b1end \n\t"         //    -- cycle 1-2-3
         "b1zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b1end: \n\t"             //    -- 4 more cycle before high
           

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b2zero \n\t"
         "b2one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b2end \n\t"         //    -- cycle 1-2-3
         "b2zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b2end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b3zero \n\t"
         "b3one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b3end \n\t"         //    -- cycle 1-2-3
         "b3zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b3end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b4zero \n\t"
         "b4one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b4end \n\t"         //    -- cycle 1-2-3
         "b4zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b4end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b5zero \n\t"
         "b5one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b5end \n\t"         //    -- cycle 1-2-3
         "b5zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b5end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b6zero \n\t"
         "b6one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b6end \n\t"         //    -- cycle 1-2-3
         "b6zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b6end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b7zero \n\t"
         "b7one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b7end \n\t"         //    -- cycle 1-2-3
         "b7zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b7end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b8zero \n\t"
         "b8one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b8end \n\t"         //    -- cycle 1-2-3
         "b8zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b8end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b9zero \n\t"
         "b9one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b9end \n\t"         //    -- cycle 1-2-3
         "b9zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b9end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b10zero \n\t"
         "b10one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b10end \n\t"         //    -- cycle 1-2-3
         "b10zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b10end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b11zero \n\t"
         "b11one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b11end \n\t"         //    -- cycle 1-2-3
         "b11zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b11end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b12zero \n\t"
         "b12one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b12end \n\t"         //    -- cycle 1-2-3
         "b12zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b12end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b13zero \n\t"
         "b13one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b13end \n\t"         //    -- cycle 1-2-3
         "b13zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b13end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b14zero \n\t"
         "b14one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b14end \n\t"         //    -- cycle 1-2-3
         "b14zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b14end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b15zero \n\t"
         "b15one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b15end \n\t"         //    -- cycle 1-2-3
         "b15zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b15end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b16zero \n\t"
         "b16one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b16end \n\t"         //    -- cycle 1-2-3
         "b16zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b16end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b17zero \n\t"
         "b17one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b17end \n\t"         //    -- cycle 1-2-3
         "b17zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b17end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b18zero \n\t"
         "b18one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b18end \n\t"         //    -- cycle 1-2-3
         "b18zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b18end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b19zero \n\t"
         "b19one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b19end \n\t"         //    -- cycle 1-2-3
         "b19zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b19end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b20zero \n\t"
         "b20one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b20end \n\t"         //    -- cycle 1-2-3
         "b20zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b20end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b21zero \n\t"
         "b21one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b21end \n\t"         //    -- cycle 1-2-3
         "b21zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b21end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b22zero \n\t"
         "b22one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "lsr %6,%6,#1 \n\t"   //    -- cycle 3 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "b b22end \n\t"         //    -- cycle 1-2-3
         "b22zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "lsr %6,%6,#1 \n\t"   //    -- cycle 1 anticipate next bit extract ( v = v >> 1)
           "nop \n\t"             //    -- cycle 2
           "nop \n\t"             //    -- cycle 3
           "nop \n\t"             //    -- cycle 4
           "nop \n\t"             //    -- cycle 5
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
        "b22end: \n\t"             //    -- 4 more cycle before high

         "mov %7,#1 \n\t"         //    if ( v & 1 == 0 )
         "and %7,%6 \n\t"
         "str %5,[%0,#0] \n\t"  //    *outset = _p
         "cmp %7,#0 \n\t"      
         "beq b23zero \n\t"
         "b23one: \n\t"            // case 1 ( 11-15 cycle high then 5 - 9 cycle low )
           "add %2,%2,#4 \n\t"    //    -- cycle 3 - anticipation _ptbits++
           "ldr %6,[%2,#0] \n\t"  //    -- cycle 4-5 - anticipation v = *ptbits
           "mov %7,#1 \n\t"       //    -- cycle 6 - anticipation if ( v & 1 == 0 )
           "and %7,%6 \n\t"       //    -- cycle 7 - anticipation
           "nop \n\t"             //    -- cycle 8
           "nop \n\t"             //    -- cycle 9
           "nop \n\t"             //    -- cycle 10
           "nop \n\t"             //    -- cycle 11
           "str %5,[%1,#0] \n\t"  //    -- cycle 12-13
           "cmp %2,%3 \n\t"       //    -- cycle 1 - check end of wh01 loop
           "bhs ewh01 \n\t"       //    -- cycle 2 - if yes goes out
           "b wh01_optim \n\t"    //    -- cycle 3-4-5 + 2 before signal goes up
         "b23zero: \n\t"           // case 0 ( 4-8 cycles high then 12-16 cycles low )       
           "str %5,[%1,#0] \n\t"  //    -- cycle 5-6
           "add %2,%2,#4 \n\t"    //    -- cycle 1 - anticipation _ptbits++
           "ldr %6,[%2,#0] \n\t"  //    -- cycle 2-3 - anticipation v = *ptbits
           "mov %7,#1 \n\t"       //    -- cycle 4 - anticipation if ( v & 1 == 0 )
           "and %7,%6 \n\t"       //    -- cycle 5 - anticipation
           "nop \n\t"             //    -- cycle 6
           "nop \n\t"             //    -- cycle 7
           "cmp %2,%3 \n\t"       //    -- cycle 8 - check end of wh01 loop
           "bhs ewh01 \n\t"       //    -- cycle 9 - if yes goes out
           "b wh01_optim \n\t"    //    -- cycle 10-11-12 + 2 before signal goes up

       "ewh01: \n\t" 
       "pop {r0,r1,r2,r3,r4,r5,r6,r7}\n\t" 
    ::
    "r" (outset),   // %0
    "r" (outclr),   // %1
    "r" (_ptbits),  // %2
    "r" (_end),     // %3
    "r" (_max),     // %4
    "r" (_p),       // %5
    "r" (__v),      // %6
    "r" (__i)       // %7
 );
  
    interrupts();
    RFduinoBLE.begin();     
    delay(200);
}


void rgb123_256_init( uint32_t pin ) {
   pinMode(pin, OUTPUT);
   // 50 us low voltage for reset
   NRF_GPIO->OUTCLR = ( 1 << pin );
}
